@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Free42 -- an HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2023  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -36,6 +36,7 @@ int docmd_insr(arg_struct *arg) {
     vartype *m, *newx;
     vartype_realmatrix *rm;
     vartype_complexmatrix *cm;
+    vartype_list *list;
     int4 rows, columns, i;
     int err, refcount;
     int interactive;
@@ -55,7 +56,9 @@ int docmd_insr(arg_struct *arg) {
     }
     if (m == NULL)
         return ERR_NONEXISTENT;
-    if (m->type != TYPE_REALMATRIX && m->type != TYPE_COMPLEXMATRIX)
+    if (m->type != TYPE_REALMATRIX
+            && m->type != TYPE_COMPLEXMATRIX
+            && m->type != TYPE_LIST)
         return ERR_INVALID_TYPE;
 
     interactive = matedit_mode == 2 || matedit_mode == 3;
@@ -75,7 +78,7 @@ int docmd_insr(arg_struct *arg) {
             if (newx == NULL)
                 return ERR_INSUFFICIENT_MEMORY;
         }
-    } else {
+    } else if (m->type == TYPE_COMPLEXMATRIX) {
         cm = (vartype_complexmatrix *) m;
         rows = cm->rows;
         columns = cm->columns;
@@ -85,6 +88,16 @@ int docmd_insr(arg_struct *arg) {
             if (newx == NULL)
                 return ERR_INSUFFICIENT_MEMORY;
         }
+    } else {
+        list = (vartype_list *) m;
+        rows = list->size;
+        columns = 1;
+        refcount = list->array->refcount;
+        if (interactive) {
+            newx = new_real(0);
+            if (newx == NULL)
+                return ERR_INSUFFICIENT_MEMORY;
+        }
     }
 
     if (matedit_i >= rows)
@@ -110,20 +123,32 @@ int docmd_insr(arg_struct *arg) {
                 rm->array->is_string[i] = 0;
                 rm->array->data[i] = 0;
             }
-        } else {
+        } else if (m->type == TYPE_COMPLEXMATRIX) {
             for (i = 2 * rows * columns - 1;
                             i >= 2 * (matedit_i + 1) * columns; i--)
                 cm->array->data[i] = cm->array->data[i - 2 * columns];
             for (i = 2 * matedit_i * columns;
                             i < 2 * (matedit_i + 1) * columns; i++)
                 cm->array->data[i] = 0;
+        } else {
+            vartype *v = new_real(0);
+            if (v == NULL) {
+                free_vartype(newx);
+                return ERR_INSUFFICIENT_MEMORY;
+            }
+            memmove(list->array->data + matedit_i + 1, list->array->data + matedit_i, (rows - matedit_i - 1) * sizeof(vartype *));
+            list->array->data[matedit_i] = v;
         }
     } else {
         /* Make sure the new array is less than 2 GB,
          * so it's addressable with a signed 32-bit index */
-        double d_bytes = ((double) (rows + 1)) * ((double) columns) * sizeof(phloat);
-        if (m->type == TYPE_COMPLEXMATRIX)
-            d_bytes *= 2;
+        int esize;
+        switch (m->type) {
+            case TYPE_REALMATRIX: esize = sizeof(phloat); break;
+            case TYPE_COMPLEXMATRIX: esize = 2 * sizeof(phloat); break;
+            case TYPE_LIST: esize = sizeof(vartype *); break;
+        }
+        double d_bytes = ((double) (rows + 1)) * ((double) columns) * esize;
         if (((double) (int4) d_bytes) != d_bytes)
             return ERR_INSUFFICIENT_MEMORY;
 
@@ -169,7 +194,7 @@ int docmd_insr(arg_struct *arg) {
             rm->array->refcount--;
             rm->array = array;
             rm->rows++;
-        } else {
+        } else if (m->type == TYPE_COMPLEXMATRIX) {
             complexmatrix_data *array = (complexmatrix_data *)
                                 malloc(sizeof(complexmatrix_data));
             if (array == NULL) {
@@ -195,6 +220,38 @@ int docmd_insr(arg_struct *arg) {
             cm->array->refcount--;
             cm->array = array;
             cm->rows++;
+        } else {
+            list_data *array = (list_data *) malloc(sizeof(list_data));
+            if (array == NULL) {
+                if (interactive)
+                    free_vartype(newx);
+                return ERR_INSUFFICIENT_MEMORY;
+            }
+            array->data = (vartype **) malloc(newsize * sizeof(vartype *));
+            if (array->data == NULL) {
+                if (interactive)
+                    free_vartype(newx);
+                free(array);
+                return ERR_INSUFFICIENT_MEMORY;
+            }
+            for (int4 i = 0; i < newsize; i++) {
+                array->data[i] = i < matedit_i ? dup_vartype(list->array->data[i])
+                               : i > matedit_i ? dup_vartype(list->array->data[i - 1])
+                               : new_real(0);
+                if (array->data[i] == NULL) {
+                    for (int4 j = 0; j < i; j++)
+                        free_vartype(array->data[j]);
+                    if (interactive)
+                        free_vartype(newx);
+                    free(array->data);
+                    free(array);
+                    return ERR_INSUFFICIENT_MEMORY;
+                }
+            }
+            array->refcount = 1;
+            list->array->refcount--;
+            list->array = array;
+            list->size++;
         }
     }
     if (interactive) {
@@ -437,6 +494,9 @@ int docmd_rclel(arg_struct *arg) {
         int4 n = matedit_i * cm->columns + matedit_j;
         v = new_complex(cm->array->data[2 * n],
                         cm->array->data[2 * n + 1]);
+    } else if (m->type == TYPE_LIST) {
+        vartype_list *list = (vartype_list *) m;
+        v = dup_vartype(list->array->data[matedit_i]);
     } else
         return ERR_INVALID_TYPE;
     if (v == NULL)
@@ -751,7 +811,9 @@ int docmd_stoel(arg_struct *arg) {
     if (m == NULL)
         return ERR_NONEXISTENT;
 
-    if (m->type != TYPE_REALMATRIX && m->type != TYPE_COMPLEXMATRIX)
+    if (m->type != TYPE_REALMATRIX
+            && m->type != TYPE_COMPLEXMATRIX
+            && m->type != TYPE_LIST)
         /* Should not happen, but could, as long as I don't implement
          * matrix locking.
          */
@@ -776,7 +838,7 @@ int docmd_stoel(arg_struct *arg) {
             return ERR_NONE;
         } else
             return ERR_INVALID_TYPE;
-    } else /* m->type == TYPE_COMPLEXMATRIX */ {
+    } else if (m->type == TYPE_COMPLEXMATRIX) {
         vartype_complexmatrix *cm = (vartype_complexmatrix *) m;
         int4 n = matedit_i * cm->columns + matedit_j;
         if (stack[sp]->type == TYPE_REAL) {
@@ -792,6 +854,14 @@ int docmd_stoel(arg_struct *arg) {
             return ERR_ALPHA_DATA_IS_INVALID;
         else
             return ERR_INVALID_TYPE;
+    } else /* m->type == TYPE_LIST */ {
+        vartype_list *list = (vartype_list *) m;
+        vartype *v = dup_vartype(stack[sp]);
+        if (v == NULL)
+            return ERR_INSUFFICIENT_MEMORY;
+        free_vartype(list->array->data[matedit_i]);
+        list->array->data[matedit_i] = v;
+        return ERR_NONE;
     }
 }
 
@@ -848,6 +918,10 @@ int docmd_stoij(arg_struct *arg) {
         vartype_complexmatrix *cm = (vartype_complexmatrix *) m;
         if (i == 0 || i > cm->rows || j == 0 || j > cm->columns)
             return ERR_DIMENSION_ERROR;
+    } else if (m->type == TYPE_LIST) {
+        vartype_list *list = (vartype_list *) m;
+        if (i == 0 || i > list->size || j != 1)
+            return ERR_DIMENSION_ERROR;
     } else
         /* Should not happen, but could, as long as I don't implement
          * matrix locking. */
@@ -991,6 +1065,7 @@ static int matedit_move(int direction) {
     vartype *m, *v;
     vartype_realmatrix *rm;
     vartype_complexmatrix *cm;
+    vartype_list *list;
     int4 rows, columns, new_i, new_j, old_n, new_n;
     int edge_flag = 0;
     int end_flag = 0;
@@ -1020,6 +1095,10 @@ static int matedit_move(int direction) {
         cm = (vartype_complexmatrix *) m;
         rows = cm->rows;
         columns = cm->columns;
+    } else if (m->type == TYPE_LIST) {
+        list = (vartype_list *) m;
+        rows = list->size;
+        columns = 1;
     } else
         return ERR_INVALID_TYPE;
 
@@ -1115,7 +1194,7 @@ static int matedit_move(int direction) {
             free_vartype(v);
             return ERR_INVALID_TYPE;
         }
-    } else /* m->type == TYPE_COMPLEXMATRIX */ {
+    } else if (m->type == TYPE_COMPLEXMATRIX) {
         if (old_n != new_n) {
             v = new_complex(cm->array->data[2 * new_n],
                             cm->array->data[2 * new_n + 1]);
@@ -1136,6 +1215,23 @@ static int matedit_move(int direction) {
             return stack[sp]->type == TYPE_STRING ? ERR_ALPHA_DATA_IS_INVALID
                                               : ERR_INVALID_TYPE;
         }
+    } else /* m->type == TYPE_LIST */ {
+        if (old_n != new_n) {
+            v = dup_vartype(list->array->data[new_n]);
+            if (v == NULL)
+                return ERR_INSUFFICIENT_MEMORY;
+        }
+        if (sp == -1) {
+            /* There's nothing to store, so leave cell unchanged */
+        } else {
+            vartype *nv = dup_vartype(stack[sp]);
+            if (nv == NULL) {
+                free_vartype(v);
+                return ERR_INSUFFICIENT_MEMORY;
+            }
+            free_vartype(list->array->data[old_n]);
+            list->array->data[old_n] = nv;
+        }
     }
 
     matedit_i = new_i;
