@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Free42 -- an HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2023  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -595,40 +595,6 @@ static int mappable_sqrt_r(phloat x, phloat *y) {
     }
 }
 
-static int mappable_sqrt_c(phloat xre, phloat xim, phloat *yre, phloat *yim) {
-    if (xre == 0 && xim == 0) {
-        *yre = 0;
-        *yim = 0;
-        return ERR_NONE;
-    }
-
-    phloat r = hypot(xre, xim);
-    phloat a = sqrt((r + fabs(xre)) / 2);
-    phloat b = xim / (a * 2);
-
-    if (p_isinf(a)) {
-        xre /= 100;
-        xim /= 100;
-        r = hypot(xre, xim);
-        a = sqrt((r + fabs(xre)) / 2);
-        b = xim / (a * 2);
-        a *= 10;
-        b *= 10;
-    }
-
-    if (xre >= 0) {
-        *yre = a;
-        *yim = b;
-    } else if (b >= 0) {
-        *yre = b;
-        *yim = a;
-    } else {
-        *yre = -b;
-        *yim = -a;
-    }
-    return ERR_NONE;
-}
-
 int docmd_sqrt(arg_struct *arg) {
     if (stack[sp]->type == TYPE_REAL) {
         phloat x = ((vartype_real *) stack[sp])->x;
@@ -645,7 +611,7 @@ int docmd_sqrt(arg_struct *arg) {
         return ERR_NONE;
     } else {
         vartype *v;
-        int err = map_unary(stack[sp], &v, mappable_sqrt_r, mappable_sqrt_c);
+        int err = map_unary(stack[sp], &v, mappable_sqrt_r, math_sqrt);
         if (err != ERR_NONE)
             return err;
         unary_result(v);
@@ -766,14 +732,22 @@ int docmd_y_pow_x(arg_struct *arg) {
     int inf;
     vartype *res;
 
-    if (stack[sp]->type == TYPE_STRING || stack[sp - 1]->type == TYPE_STRING)
+    if (false) {
+        done:
+        if (res == NULL)
+            return ERR_INSUFFICIENT_MEMORY;
+        else
+            return binary_result(res);
+    }
+
+    if (stack[sp]->type == TYPE_STRING || stack[sp - 1]->type == TYPE_STRING) {
         return ERR_ALPHA_DATA_IS_INVALID;
-    else if (stack[sp]->type != TYPE_REAL
+    } else if (stack[sp]->type != TYPE_REAL
             && stack[sp]->type != TYPE_COMPLEX
             || stack[sp - 1]->type != TYPE_REAL
-            && stack[sp - 1]->type != TYPE_COMPLEX)
+            && stack[sp - 1]->type != TYPE_COMPLEX) {
         return ERR_INVALID_TYPE;
-    else if (stack[sp]->type == TYPE_REAL) {
+    } else if (stack[sp]->type == TYPE_REAL) {
         phloat x = ((vartype_real *) stack[sp])->x;
         if (x == floor(x)) {
             /* Integer exponent */
@@ -794,19 +768,22 @@ int docmd_y_pow_x(arg_struct *arg) {
                     r = inf < 0 ? NEG_HUGE_PHLOAT : POS_HUGE_PHLOAT;
                 }
                 res = new_real(r);
-                if (res == NULL)
-                    return ERR_INSUFFICIENT_MEMORY;
-                else
-                    return binary_result(res);
+                goto done;
             } else {
                 /* Complex number to integer power */
                 phloat rre, rim, yre, yim;
                 int4 ex;
-                if (x < -2147483647.0 || x > 2147483647.0)
-                    /* For really huge exponents, the repeated-squaring
-                     * algorithm for integer exponents loses its accuracy
-                     * and speed advantage, and we switch to the general
-                     * complex-to-real-power code instead.
+                if (x < -256.0 || x > 256.0)
+                    /* At some point, the cumulative error in repeated squaring
+                     * becomes larger than the error in the polar power
+                     * formula, so we only use repeated squaring for smallish
+                     * exponents. The limit of 256 is somewhat arbitrary, but
+                     * high enough that whenever possible, the function will
+                     * return exact results for short bases, i.e. bases where
+                     * first and last nonzero digits are close together. The
+                     * smallest non-trivial base is 1+i, which can be raised to
+                     * the power 225 in the decimal version, and to the power
+                     * 107 in the binary version, with exact results.
                      */
                     goto complex_pow_real_1;
                 rre = 1;
@@ -816,26 +793,22 @@ int docmd_y_pow_x(arg_struct *arg) {
                 ex = to_int4(x);
                 if (ex <= 0 && yre == 0 && yim == 0)
                     return ERR_INVALID_DATA;
-                if (ex < 0) {
-                    phloat h = hypot(yre, yim);
-                    yre = yre / h / h;
-                    yim = (-yim) / h / h;
+                bool invert = ex < 0;
+                if (invert)
                     ex = -ex;
-                }
                 while (1) {
                     phloat tmp;
                     if ((ex & 1) != 0) {
                         tmp = rre * yre - rim * yim;
                         rim = rre * yim + rim * yre;
                         rre = tmp;
-                        /* TODO: can one component be infinite while
-                         * the other is zero? If yes, how do we handle
-                         * that?
-                         */
-                        if (p_isinf(rre) && p_isinf(rim))
-                            break;
+                        if (p_isinf(rre) || p_isnan(rre) || p_isinf(rim) || p_isnan(rim))
+                            goto complex_pow_real_1;
                         if (rre == 0 && rim == 0)
-                            break;
+                            if (invert)
+                                goto complex_pow_real_1;
+                            else
+                                break;
                     }
                     ex >>= 1;
                     if (ex == 0)
@@ -843,6 +816,13 @@ int docmd_y_pow_x(arg_struct *arg) {
                     tmp = yre * yre - yim * yim;
                     yim = 2 * yre * yim;
                     yre = tmp;
+                    if (p_isinf(yre) || p_isnan(yre) || p_isinf(yim) || p_isnan(yim))
+                        goto complex_pow_real_1;
+                }
+                if (invert) {
+                    phloat h = hypot(rre, rim);
+                    rre = rre / h / h;
+                    rim = (-rim) / h / h;
                 }
                 if ((inf = p_isinf(rre)) != 0) {
                     if (!flags.f.range_error_ignore)
@@ -855,10 +835,7 @@ int docmd_y_pow_x(arg_struct *arg) {
                     rim = inf < 0 ? NEG_HUGE_PHLOAT : POS_HUGE_PHLOAT;
                 }
                 res = new_complex(rre, rim);
-                if (res == NULL)
-                    return ERR_INSUFFICIENT_MEMORY;
-                else
-                    return binary_result(res);
+                goto done;
             }
         } else if (stack[sp - 1]->type == TYPE_REAL) {
             /* Real number to noninteger real power */
@@ -879,10 +856,7 @@ int docmd_y_pow_x(arg_struct *arg) {
                 r = inf < 0 ? NEG_HUGE_PHLOAT : POS_HUGE_PHLOAT;
             }
             res = new_real(r);
-            if (res == NULL)
-                return ERR_INSUFFICIENT_MEMORY;
-            else
-                return binary_result(res);
+            goto done;
         } else {
             /* Complex (or negative real) number to noninteger real power */
             complex_pow_real_1:
@@ -917,10 +891,7 @@ int docmd_y_pow_x(arg_struct *arg) {
                 rim = yr * im;
             }
             res = new_complex(rre, rim);
-            if (res == NULL)
-                return ERR_INSUFFICIENT_MEMORY;
-            else
-                return binary_result(res);
+            goto done;
         }
     } else {
         /* Real or complex number to complex power */
@@ -942,10 +913,7 @@ int docmd_y_pow_x(arg_struct *arg) {
                 return ERR_INVALID_DATA;
             else
                 res = new_complex(0, 0);
-            if (res == NULL)
-                return ERR_INSUFFICIENT_MEMORY;
-            else
-                return binary_result(res);
+            goto done;
         }
         err = mappable_ln_c(yre, yim, &lre, &lim);
         if (err != ERR_NONE)
@@ -957,10 +925,7 @@ int docmd_y_pow_x(arg_struct *arg) {
         if (err != ERR_NONE)
             return err;
         res = new_complex(xre, xim);
-        if (res == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return binary_result(res);
+        goto done;
     }
 }
 
@@ -1090,5 +1055,8 @@ int docmd_stoflag(arg_struct *arg) {
     if (new_g != -1 || new_rad != -1)
         shell_annunciators(-1, -1, -1, -1, new_g, new_rad);
 
+    if (b <= 30 && e >= 30 && ((lfs >> 30) & 1) == 1)
+        mode_disable_stack_lift = true;
+
     return ERR_NONE;
 }
